--
-- Brush
--
-- @package 	fs19.bresselUndLade_brush.specializations.brush
-- @auhtor 		IceBlade <www.cg-production.com>
-- @copyright 	2020 CG-Production
-- @license    	www.cg-production.com/license/FS_Script_Licence_2018_1.txt
-- @version		1.0.0.0
-- @history		<1.0.0.0> <14.04.2020> creation
--

Brush = {};
Brush.MOD_NAME = g_currentModName;

function Brush.prerequisitesPresent(specializations)
    return true;
end;

function Brush.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "processBrushArea", Brush.processBrushArea)
    SpecializationUtil.registerFunction(vehicleType, "brushIsUsable", Brush.brushIsUsable)
    SpecializationUtil.registerFunction(vehicleType, "updateAnimationNodes", Brush.updateAnimationNodes)
end;

function Brush.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", Brush);
    SpecializationUtil.registerEventListener(vehicleType, "onUpdate", Brush);
    SpecializationUtil.registerEventListener(vehicleType, "onStartWorkAreaProcessing", Brush);
    SpecializationUtil.registerEventListener(vehicleType, "onEndWorkAreaProcessing", Brush);
end;

function Brush:onLoad(savegame)
    self.spec_brush = self[("spec_%s.brush"):format(Brush.MOD_NAME)]
    local spec = self.spec_brush;

    local baseKey = "vehicle.brush";

    spec.speedCheckFactor = Utils.getNoNil(getXMLFloat(self.xmlFile, baseKey .. ".speed#factor"), 0);
    spec.isProcessing = false;
    spec.isPlayingSound = false;
    spec.sound = nil;

    if self.isClient then
        spec.sound = g_soundManager:loadSampleFromXML(self.xmlFile, baseKey .. ".sounds", "brush", self.baseDirectory, self.components, 0, AudioGroup.VEHICLE, self.i3dMappings, self);
    end;

    spec.animationNodes = {};
    spec.resetAnimation = true;
    spec.lastAnimationValue = 0;

    local i = 0;
    while true do
        local key = (baseKey .. ".animationNodes.animationNode(%d)"):format(i);

        if not hasXMLProperty(self.xmlFile, key) then
            break;
        end;

        local animationNode = {};
        animationNode.node = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, key .. "#node"), self.i3dMappings);
        animationNode.minRot = math.rad(Utils.getNoNil(getXMLFloat(self.xmlFile, key .. "#minRot"), 0));
        animationNode.maxRot = math.rad(Utils.getNoNil(getXMLFloat(self.xmlFile, key .. "#maxRot"), 0));

        table.insert(spec.animationNodes, animationNode);

        i = i + 1;
    end;
end;

function Brush:onUpdate(dt)
    local spec = self.spec_brush;

    if self.isClient then
        if spec.isProcessing then
            if self:brushIsUsable() then
                local animationValue = self:getLastSpeed();
                local isNevative = false;
                --[[if self.movingDirection > 0 then
                    animationValue = 9;
                end;

                if self.movingDirection < 0 then
                    animationValue = -9;
                end;]]

                if self.movingDirection < 0 then
                    animationValue = animationValue - (2* animationValue);
                    isNevative = true;
                end;

                if not isNevative and animationValue >= spec.lastAnimationValue then
                    self:updateAnimationNodes(animationValue);
                    spec.resetAnimation = true;
                    spec.lastAnimationValue = animationValue;
                end;

                if isNevative and animationValue <= spec.lastAnimationValue then
                    self:updateAnimationNodes(animationValue);
                    spec.resetAnimation = true;
                    spec.lastAnimationValue = animationValue;
                end;
            end;
        else
            if spec.resetAnimation then
                self:updateAnimationNodes(0);
                spec.resetAnimation = false;
                spec.lastAnimationValue = 0;
            end;
        end;
    end;
end;

function Brush:updateAnimationNodes(value)
    local spec = self.spec_brush;

    for _, node in pairs(spec.animationNodes) do
        if value > node.maxRot then
            value = node.maxRot;
        end;

        if value < node.minRot then
            value = node.minRot;
        end;

        setRotation(node.node, value, 0, 0);
    end;
end;

function Brush:processBrushArea(workarea, dt)
    local spec = self.spec_brush;

    if self:brushIsUsable() then
        local startX,_,startZ = getWorldTranslation(workarea.start);
        local widthX,_,widthZ = getWorldTranslation(workarea.width);
        local heightX,_,heightZ = getWorldTranslation(workarea.height);

        FSDensityMapUtil.eraseTireTrack(startX, startZ, widthX, widthZ, heightX, heightZ);

        spec.isProcessing = true;
    end;

    return 0, 0;
end;

function Brush:brushIsUsable()
    local spec = self.spec_brush;
    return self:getLastSpeed() > spec.speedCheckFactor
end;

function Brush:onStartWorkAreaProcessing(dt)
    local spec = self.spec_brush;
    spec.isProcessing = false;
end;

function Brush:onEndWorkAreaProcessing(dt, hasProcessed)
    local spec = self.spec_brush;

    if self.isClient then
        if spec.isProcessing then
            if not spec.isPlayingSound then
                g_soundManager:playSample(spec.sound)
                spec.isPlayingSound = false;
            end;
        else
            g_soundManager:stopSample(spec.sound)
            spec.isPlayingSound = false;
        end;
    end;
end;